#include "LtrResolver.h"
#include "lqmeas_config.h"
#include "lqmeas/devs/ltr/crates/LtrCrate.h"
#include "lqmeas/devs/DeviceValidator.h"
#include "ltr/include/ltrapi.h"



#include <QtDebug>

#ifdef LQMEAS_DEV_LTR24
    #include "modules/ltr24.h"
#endif
#ifdef LQMEAS_DEV_LTR25
    #include "modules/ltr25.h"
#endif
#ifdef LQMEAS_DEV_LTR35
    #include "modules/ltr35.h"
#endif
#ifdef LQMEAS_DEV_LTR51
    #include "modules/ltr51.h"
#endif
#ifdef LQMEAS_DEV_LTR210
    #include "modules/ltr210.h"
#endif


namespace LQMeas {
    QList<QSharedPointer<LtrModule> > LtrResolver::getModuleList(QStringList nameFilter) {
        QScopedPointer<DeviceValidator> val(new DeviceNameValidator(nameFilter));
        return getModuleList(val.data());
    }

    QList<QSharedPointer<LtrModule> > LtrResolver::getModuleList(const DeviceValidator *validator) {
        QList<QSharedPointer<LtrModule> > fndList;

        QList<QSharedPointer<LtrCrate> > crates = getCratesList();
        foreach (QSharedPointer<LtrCrate> crate, crates) {
            fndList.append(crate->getModuleList(validator));
        }
        return fndList;
    }

    QList<QSharedPointer<LtrCrate> > LtrResolver::getCratesList() {
        TLTR srv;
        INT err;

        QList<QSharedPointer<LtrCrate> >  ret;

        LTR_Init(&srv);
        srv.cc = LTR_CC_CHNUM_CONTROL;
        strcpy(srv.csn, LTR_CSN_SERVER_CONTROL);

        err = LTR_Open(&srv);
        if (err==LTR_OK) {
            char serials[LTR_CRATES_MAX][LTR_CRATE_SERIAL_SIZE];
            err = LTR_GetCrates(&srv, (BYTE*)serials);
            if (err==LTR_OK) {
                for (unsigned i=0; i < LTR_CRATES_MAX; i++) {
                    if (serials[i][0]!='\0') {
                        TLTR chnd;
                        INT crate_err = 0;
                        LTR_Init(&chnd);
                        chnd.cc = LTR_CC_CHNUM_CONTROL;
                        memcpy(chnd.csn, serials[i], LTR_CRATE_SERIAL_SIZE);
                        crate_err = LTR_Open(&chnd);
                        if (crate_err==LTR_OK) {
                            TLTR_CRATE_INFO crate_info;
                            QSharedPointer<LtrCrate> crate;
                            QString ctype;
                            unsigned slot_cnt;
                            QString serial = QString::fromLatin1(serials[i]);
                            crate_err = LTR_GetCrateInfo(&chnd, &crate_info);

                            LTR_Close(&chnd);

                            if (crate_err == LTR_OK) {
                                crate = getCrate(serial);
                                if (!crate) {
                                    crate = QSharedPointer<LtrCrate>(
                                            new LtrCrate(chnd, serial, crate_info));
                                    m_crates.append(crate);
                                }
                                ret.append(crate);
                            }
                        }
                    }
                }
            }
            LTR_Close(&srv);
        }
        return ret;
    }


    QList<QSharedPointer<Device> > LtrResolver::getDevList() {
        QList<QSharedPointer<LtrCrate> > crates = getCratesList();
        QList<QSharedPointer<Device> > retlist;
        foreach (QSharedPointer<LtrCrate> crate, crates) {
            retlist.append(crate);
        }
        return retlist;
    }

    QSharedPointer<LtrCrate> LtrResolver::getCrate(QString serial) {
        foreach (QSharedPointer<LtrCrate> curCrate, m_crates) {
            if (curCrate->serial()==serial) {
                return curCrate;
            }
        }
        return QSharedPointer<LtrCrate>();
    }

    LtrResolver *LtrResolver::resolver() {
        static LtrResolver resolver;
        return &resolver;
    }

    LtrResolver::LtrResolver() {

    }

    QSharedPointer<LtrModule> LtrResolver::createModule(QSharedPointer<LtrCrate> crate, unsigned slot, quint16 mid) {
        switch (mid) {
    #ifdef LQMEAS_DEV_LTR24
            case LTR24::typeModuleID:
                return QSharedPointer<LtrModule>(new LTR24(crate, slot));
    #endif
    #ifdef LQMEAS_DEV_LTR25
            case LTR25::typeModuleID:
                return QSharedPointer<LtrModule>(new LTR25(crate, slot));
    #endif
    #ifdef LQMEAS_DEV_LTR35
            case LTR35::typeModuleID:
                return QSharedPointer<LtrModule>(new LTR35(crate, slot));
    #endif
    #ifdef LQMEAS_DEV_LTR51
            case LTR51::typeModuleID:
                return QSharedPointer<LtrModule>(new LTR51(crate, slot));
    #endif
    #ifdef LQMEAS_DEV_LTR210
            case LTR210::typeModuleID:
                return QSharedPointer<LtrModule>(new LTR210(crate, slot));
    #endif
        }

        return QSharedPointer<LtrModule>();
    }
}

