#ifndef LQMEAS_LTRCRATE_H
#define LQMEAS_LTRCRATE_H

#include <QObject>
#include <QStringList>
#include <QSharedPointer>
#include <QVector>
#include "ltr/include/ltrapi.h"

#include "lqmeas/devs/resolver/DevicesResolver.h"


namespace LQMeas {
    class LtrModule;
    class LtrResolver;
    class DeviceValidator;


    class LtrCrate : public Device, public DevicesResolver {
        Q_OBJECT
    public:
        ~LtrCrate();

        QString name() const {return m_typeName;}
        QString serial() const {return m_serial;}
        unsigned slotCount() const {return m_slot_cnt;}


        virtual bool isOpened() const;
        /* адрес ltrd с которым установлено соединение */
        quint32 ltrdIpAddr();
        /* порт TCP соединения с ltrd */
        quint16 ltrdTcpPort();


        DeviceInterface iface() const {return m_iface;}

        QList<QSharedPointer<LtrModule> > getModuleList(QStringList nameFilter);
        QList<QSharedPointer<LtrModule> > getModuleList(const DeviceValidator* validator);
        QList<QSharedPointer<Device> > getDevList();

        DevicesResolver *childResolver() {return dynamic_cast<DevicesResolver *>(this);}


        Error error(int err) const;
    protected:
        Error protOpen();
        Error protClose();
        Error protLoadConfig(QSettings &set);
        Error protSaveConfig(QSettings &set) const;
        Error protConfigure();


    private:
        explicit LtrCrate(TLTR hnd, QString serial, TLTR_CRATE_INFO info);


        QString m_typeName;
        QString m_serial;
        DeviceInterface m_iface;
        unsigned m_slot_cnt;

        QVector<QSharedPointer<LtrModule> > m_modules;

        mutable TLTR m_hnd;

        friend class LtrResolver;

    };
}

#endif // LQMEAS_LTRCRATE_H
