#ifndef LQMEAS_LTRMODULE_H
#define LQMEAS_LTRMODULE_H


#include <QSharedPointer>

#include "lqmeas/devs/Device.h"
#include "lqmeas/devs/ltr/crates/LtrCrate.h"
#include "ltr/include/ltrapi.h"

namespace LQMeas {
    class LtrResolver;
    class LtrFrameReceiver;

    class LtrModule : public Device {
        Q_OBJECT
    public:

        ~LtrModule();

        unsigned mid() const {return m_mid;}
        int slot() const {return m_slot; }

        QString location() const {return tr("Slot %1").arg(QString::number(slot()+1));}

        QSharedPointer<LtrCrate> crate() const {return m_crate;}


        Error error(int err) const;
    protected:
        explicit LtrModule(QSharedPointer<LtrCrate> crate, unsigned slot, unsigned mid, QObject *parent = 0);


        virtual QString errorString(int err) const;
        virtual TLTR* channel() const=0;

        virtual Error rawWordsReceive(DWORD *wrds, unsigned size, unsigned tout, unsigned *recvd_size);
        virtual Error rawWordsSend(const DWORD *wrds, unsigned size, unsigned tout, unsigned *sent_size);
    private:
        QSharedPointer<LtrCrate> m_crate;
        unsigned m_slot;
        unsigned m_mid;

        friend class LtrResolver;
        friend class LtrFrameReceiver;
    };
}

#endif // LTRMODULE_H
