#ifndef LQMEAS_LTR24_H
#define LQMEAS_LTR24_H

#include "LtrModule.h"
#include "ltr/include/ltr24api.h"
#include "lqmeas/ifaces/in/DevAdc.h"
#include "lqmeas/devs/DeviceFrameReceiver.h"


namespace LQMeas {
    class LTR24 : public LtrModule, public DevAdc {
        Q_OBJECT
    public:
        static QString typeModuleName();
        static const quint16 typeModuleID = LTR_MID_LTR24;

        struct AdcConfig {
            BYTE        ADCFreqCode;
            BYTE        DataFmt;
            BYTE        ISrcValue;
            BOOL        TestMode;
            struct {
                BOOL    Enabled;
                BYTE    Range;
                BOOL    AC;
                BOOL    ICPMode;
            } Ch[LTR24_CHANNEL_NUM];
        };


        QString serial() const;
        QString name() const;
        QString modificationName() const;
        int pldVer() const;
        bool supportICP() const;



        /* ----------------------- интерфейс ввода  --------------------------*/
        DevAdc *devAdc() {return static_cast<DevAdc*>(this);}


        const double adcRangeMaxVal(int range) const;
        const double adcRangeMinVal(int range) const;
        unsigned adcRangesCnt() const;
        double adcFreqMax() const;

        static double typeAdcFreqVal(int idx);
        static unsigned typeAdcFreqsCnt();
        double adcFreqVal(int idx) const {return typeAdcFreqVal(idx); }
        unsigned adcFreqsCnt() const {return typeAdcFreqsCnt(); }


        double adcFreq() const;
        double adcChRange(int ch) const;

        QString errorString(int err) const;
        bool isOpened() const;


        bool adcIsRunning() const;
        bool adcChEnabled(int ch) const;


        Error adcGetData(double *data, unsigned size, unsigned flags,
                         unsigned tout, unsigned *recvd_size);

        AdcConfig adcCurConfig() const {return m_cfg; }
    protected:
        TLTR* channel() const;

        Error protOpen();
        Error protClose();
        Error protConfigure();
        Error protLoadConfig(QSettings &set);
        Error protSaveConfig(QSettings &set) const;

        Error protAdcSetRawConfig(const void *rawCfg, size_t cfg_size);
        Error rawWordsReceive(DWORD *wrds, unsigned size, unsigned tout, unsigned *recvd_size);

        Error protAdcStart();
        Error protAdcStop();


    private:
        explicit LTR24(QSharedPointer<LtrCrate> crate, unsigned slot, QObject *parent = 0);
        mutable TLTR24 m_hnd;
        AdcConfig m_cfg;

        DeviceFrameReceiver<LTR24, DWORD> m_receiver;

        friend class DeviceFrameReceiver<LTR24, DWORD>;
        friend class LtrResolver;

    };
}

#endif // LQMEAS_LTR24_H
