#ifndef LQMEAS_DEVOUTSTREAM_H
#define LQMEAS_DEVOUTSTREAM_H

#include "DevOut.h"
#include "DevOutStreamThread.h"

namespace LQMeas {
    class Device;

    /***********************************************************************//**
        @brief Интерфейс для вывода аналоговых и/или цифровых сигналов в потоковом
                режиме.

        Интерфейс предоставляет функции потокового режима при этом наследуя
            функции настройки сигналов от интерфейса DevOut. Генерация
            сигнала запускается в отдельном специальном потоке.

        Типичный вариант использования:
          - настройка модуля и сигналов (см. интерфейс DevOut)
          - предзагрузка заданного количества точек в модуль с помощью outStreamInit()
          - запуск генерации потока с помощью outStreamStart()
          - при необходимости остановить генерацию вызвать protOutStreamStop()

        Для отслеживания ошибок генерации потока можно отслеживать событие
        остановки потока, полученного с помощью outStreamThread() и при остановке
        можно проверить код ошибки с помощью outStreamError()
      ************************************************************************/

    class DevOutStream : public DevOut {
    public:
        Error outStreamInit(unsigned preload_points);
        Error outStreamStart();
        Error outStreamStop(unsigned tout=30000);


        bool outStreamIsRunning() const {return m_running;}
        Error outStreamError() const {return m_streamThread.error();}


        DevOutStreamThread *outStreamThread() {return &m_streamThread;}
    protected:
        DevOutStream(Device* dev);

        virtual Error protOutStreamInit() = 0;
        virtual Error protOutStreamStart() = 0;
        virtual Error protOutStreamStop(unsigned tout) = 0;
    private:
        bool m_running;

        DevOutStreamThread m_streamThread;
        Device *m_dev;


        friend class DevOutStreamThread;
    };
}

#endif // LQMEAS_DEVOUTSTREAM_H
